@@ -110,7 +110,7 @@ class Archive(object):
     def __repr__(self):
         return "Archive(%r, prepare=%r, lowmem=%r, verbose=%r)" % \
                 (self.filename, self.prepare, self.lowmem, self.verbose)
-    
+
     def __str__(self):
         return self.filename
 
@@ -132,7 +132,7 @@ class Archive(object):
             print("IOError: Filename not found")
             raise SystemExit
         self.header = hdulist[0].header
-        self.keys = fmap(lambda x: x.name, hdulist)
+        self.keys = [hdu.name for hdu in hdulist]
         tablenames = self.keys[:] #temporary list for checking other tables
 
         if 'HISTORY' in self.keys:
@@ -153,10 +153,10 @@ class Archive(object):
             self.paramheader = dict()
             for key in self.paramheaderlist:
                 self.paramheader[key] = hdulist['PSRPARAM'].header[key]
-            self.params = Par(fmap(lambda x: x[0], hdulist['PSRPARAM'].data), numwrap=float)
+            self.params = Par([elem[0] for elem in hdulist['PSRPARAM'].data], numwrap=float)
         elif 'PSREPHEM' in self.keys:
             tablenames.remove('PSREPHEM')
-            paramkeys = fmap(lambda x: x.name, hdulist['PSREPHEM'].columns)
+            paramkeys = [col.name for col in hdulist['PSREPHEM'].columns]
             paramvals = hdulist['PSREPHEM'].data[0]
             paramstrs = fmap(lambda x, y: "%s %s"%(x, y), paramkeys, paramvals)
             #self.paramheaderlist = hdulist['PSREPHEM'].header.keys()
@@ -195,7 +195,7 @@ class Archive(object):
             raise ValueError("This is a fluxcal file: currently not implemented")
 
         self.subintinfo = dict()
-        self.subintinfolist = fmap(lambda x: x.name, hdulist['SUBINT'].columns[:-5])
+        self.subintinfolist = [elem.name for elem in hdulist['SUBINT'].columns[:-5]]
         for i, column in enumerate(hdulist['SUBINT'].columns[:-5]):
             self.subintinfo[column.name] = (column.format,
                                             column.unit,
@@ -209,7 +209,7 @@ class Archive(object):
             self.weights = None
             self._data = None
             return
-            
+
         DATA = hdulist['SUBINT'].data['DATA']
         if np.ndim(DATA) == 5:
             DATA = DATA[:, 0, :, :, :] #remove the nsblk column
@@ -298,7 +298,7 @@ class Archive(object):
         elif nsubint == 1 and npol == 1:
             for k in K:
                 self._data[0, 0, k, :] = (DAT_SCL[0, k]*DATA[0, 0, k, :]+DAT_OFFS[0, k])#*DAT_WTS[0, k] #dat WTS[0]?
-        elif nsubint == 1 and nchan == 1:              
+        elif nsubint == 1 and nchan == 1:
             for j in J:
                 self._data[0, j, 0, :] = (DAT_SCL[0, j]*DATA[0, j, 0, :]+DAT_OFFS[0, j])#*DAT_WTS[0]
         elif npol == 1 and nchan == 1:
@@ -482,7 +482,7 @@ class Archive(object):
         cols.append(pyfits.Column(name='DAT_WTS', format='%iE'%np.shape(self.weights)[1], array=self.weights)) #call getWeights()
 
         nsubint, npol, nchan, nbin = self.shape(squeeze=False)
-        
+
         DAT_OFFS = np.zeros((nsubint, npol*nchan), dtype=np.float32)
         DAT_SCL = np.zeros((nsubint, npol*nchan), dtype=np.float32)
         DATA = self.getData(squeeze=False, weight=False)
@@ -598,14 +598,14 @@ class Archive(object):
 
         newnsubint = nsubint//factor
         for i in xrange(newnsubint):
-            weightretval[i, :] = np.mean(self.weights[i*factor:(i+1)*factor, :], axis=0)
+            weightretval[i, :] = np.nanmean(self.weights[i*factor:(i+1)*factor, :], axis=0)
 
         for i in xrange(newnsubint):
-            newdurations[i] += np.sum(self.durations[i*factor:(i+1)*factor])
+            newdurations[i] += np.nansum(self.durations[i*factor:(i+1)*factor])
             for j in xrange(npol):
                 for k in xrange(nchan):
                     for l in xrange(nbin):
-                        retval[i, j, k, l] = np.sum(self.data[i*factor:(i+1)*factor, j, k, l] * self.weights[i*factor:(i+1)*factor, k]) / np.sum(self.weights[i*factor:(i+1)*factor, k])
+                        retval[i, j, k, l] = np.nansum(self.data[i*factor:(i+1)*factor, j, k, l] * self.weights[i*factor:(i+1)*factor, k]) / np.nansum(self.weights[i*factor:(i+1)*factor, k])
 
         self.weights = weightretval
         self.data = retval
@@ -663,7 +663,7 @@ class Archive(object):
         self.record(inspect.currentframe())
 
         nsubint, npol, nchan, nbin = self.shape(squeeze=False)
-        
+
         retval = np.zeros((nsubint, npol, len(np.r_[0:nchan:factor]), nbin))
 
         weightretval = np.zeros((nsubint, len(np.r_[0:nchan:factor])))
@@ -679,15 +679,15 @@ class Archive(object):
         newfreq = np.zeros(newnchan) #this will only be 1D
 
         for k in xrange(newnchan):
-            weightretval[:, k] = np.mean(self.weights[:, k*factor:(k+1)*factor], axis=1)
-            newfreq[k] = np.sum(freq[k*factor:(k+1)*factor])/float(factor) #unweighted!
+            weightretval[:, k] = np.nanmean(self.weights[:, k*factor:(k+1)*factor], axis=1)
+            newfreq[k] = np.nansum(freq[k*factor:(k+1)*factor])/float(factor) #unweighted!
 
         for i in xrange(nsubint):
             for j in xrange(npol):
                 for k in xrange(newnchan):
                     for l in xrange(nbin):
                         retval[i, j, k, l] = \
-                        np.sum(self.data[i, j, k*factor:(k+1)*factor, l] * self.weights[i, k*factor:(k+1)*factor]) / np.sum(self.weights[i, k*factor:(k+1)*factor])
+                        np.nansum(self.data[i, j, k*factor:(k+1)*factor, l] * self.weights[i, k*factor:(k+1)*factor]) / np.nansum(self.weights[i, k*factor:(k+1)*factor])
 
 
         self.weights = weightretval
@@ -708,7 +708,7 @@ class Archive(object):
         else:
             self.record(inspect.currentframe())
             nsubint, npol, nchan, nbin = self.shape(squeeze=False)
-            
+
             retval = np.zeros((nsubint, npol, nchan, len(np.r_[0:nbin:factor])))
             counts = np.zeros_like(retval)
             for i in xrange(factor):
@@ -827,7 +827,7 @@ class Archive(object):
             self.template = SP.SinglePulse(template, windowsize=int(self.getNbin()//8))
             self.opw = self.template.opw
             self.template.normalize()
-            
+
         return self.template
 
     def superprep(self):
@@ -884,7 +884,7 @@ class Archive(object):
         self.average_profile = np.roll(self.average_profile, diff)
         tbin = self.getTbin()
         if tbin is None: #error catching
-            return 
+            return
         self.channel_delays += Decimal(str(-1*diff*tbin)) #this is unnecessary? FIX THIS
         self.calculateOffpulseWindow()
         return self
@@ -952,7 +952,7 @@ class Archive(object):
         fd_poln = self.header['FD_POLN']
         return Calibrator(freqs, caldata, calerrs, pol_type=pol_type, fd_poln=fd_poln, Funit=self.getFrequencyUnit(), Sunit=self.getDataUnit(), verbose=self.verbose)
 
-    
+
     def calibrate(self, psrcalar, fluxcalonar=None, fluxcaloffar=None):
         """Calibrates using another archive"""
         self.record(inspect.currentframe())
@@ -995,7 +995,7 @@ class Archive(object):
     @data.setter
     def data(self, value):
         self._data = value
-        self.weighted_data = value*self.weights[:,None,:,None]/np.sum(self.weights)
+        self.weighted_data = value*self.weights[:,None,:,None]/np.nansum(self.weights)
 
     def getData(self, squeeze=True, setnan=None, weight=True):
         """Returns the data array,  fully squeezed"""
@@ -1009,10 +1009,10 @@ class Archive(object):
 
         if setnan is not None:
             data = np.where(data == setnan, np.nan, data)
-        
+
         #return np.copy(data) #removes pointer to data
         return data
-    
+
     def setData(self, newdata):
         """Sets the data,  very dangerous!"""
         self.record(inspect.currentframe())
@@ -1121,7 +1121,7 @@ class Archive(object):
         """Convenience function for getAxis"""
         return self.getAxis('F', **kwargs)
     getFreqs = getFrequencies
-    
+
     def getTimes(self, **kwargs):
         """Convenience function for getAxis"""
         return self.getAxis('T', **kwargs)
@@ -1251,7 +1251,7 @@ class Archive(object):
                     if windowsize >= len(sptemp.data):
                         raise IndexError("Improper window size")
                     sptemp = SP.SinglePulse(sptemp.data, windowsize=windowsize)
-                    
+
                 gs = np.zeros((fullshape[0], fullshape[2]))
                 offs = np.zeros((fullshape[0], fullshape[2]))
                 sig_gs = np.zeros((fullshape[0], fullshape[2]))
@@ -1325,10 +1325,10 @@ class Archive(object):
                 plt.show()
         else:
             raise IndexError("Invalid dimensions for plot()")
-            
+
     def imshow(self, ax=None, cbar=False, mask=None, show=True,
                filename=None, setnan=0.0, cmap=None, **kwargs):
-        """ 
+        """
         Basic imshow of data
 
         Parameters
@@ -1338,7 +1338,7 @@ class Archive(object):
         cbar : bool
             Draw a color bar
         mask : float
-            If set to a value, use a masked array and set the mask 
+            If set to a value, use a masked array and set the mask
             to that value
         show : bool
             If true, plot the image
@@ -1347,7 +1347,7 @@ class Archive(object):
         setnan : float
             Value to set NaNs to in the plot
         cmap : str or matplotlib.colors.Colormap
-            Either a matplotlib colormap object or a string of the 
+            Either a matplotlib colormap object or a string of the
             name of the colormap. Default is cividis if available,
             otherwise viridis
 
@@ -1362,7 +1362,7 @@ class Archive(object):
         if ax is None:
             fig = plt.figure()
             ax = fig.add_subplot(111)
-        
+
         if len(np.shape(data)) == 2:
             extent = None
             if shape[0] == 1 and shape[1] == 1:
@@ -1384,7 +1384,7 @@ class Archive(object):
 
 
             #cmap.set_bad(color='k', alpha=1.0)
-                
+
             if mask is not None:
                 u.imshow(ma.masked_array(data, mask=mask), ax=ax, extent=extent, cmap=cmap, **kwargs)
             else:
@@ -1416,11 +1416,11 @@ class Archive(object):
                 ax_time = ax.twiny()
                 ax_time.set_xlim(0, self.getNsubint())
                 ax_time.set_xlabel("Subintegration Number")
-                
+
         elif len(np.shape(data)) == 1:
             # Trust that this is a single subintegration in time?
             Tedges = self.getAxis('T', edges=True) #is this true?
-            extent = [0, 1, Tedges[0], Tedges[-1]]       
+            extent = [0, 1, Tedges[0], Tedges[-1]]
 
             u.imshow(data[np.newaxis, ...], ax=ax, extent=extent, cmap=cmap, **kwargs)
 
@@ -1429,8 +1429,8 @@ class Archive(object):
             ax.set_ylabel("Time (%s)"%unit)
             ax2 = ax.twinx()
             ax2.set_ylim(0, self.getNsubint())
-            ax2.set_ylabel("Subintegration Number")                
-                
+            ax2.set_ylabel("Subintegration Number")
+
         else:
             raise IndexError("Invalid dimensions for plotting")
 
@@ -1470,7 +1470,7 @@ class Archive(object):
         data = self.getData(squeeze=True)
         if len(np.shape(data)) == 2:
             if offset is None:
-                offset = np.max(np.average(data, axis=0))#*0.5# * 2.10 #?
+                offset = np.max(np.nanmean(data, axis=0))#*0.5# * 2.10 #?
 
             fig = plt.figure(figsize=(6, 6))
             if album:
@@ -1505,9 +1505,6 @@ class Archive(object):
                     #y = np.roll(y, 100*i) # for testing
 
                     ax.plot(y, color, zorder=z)
-
-                    ax.set_xlim(XLOW, XHIGH)
-                    ax.set_ylim(YLOW, YHIGH)
                     ax.fill_between(x, y, where=(y >= YLOW), color=bgcolor, zorder=z) #testing
 
             else:
@@ -1603,8 +1600,7 @@ class Archive(object):
             #print tauhatdec,np.dtype(tauhatdec)
 
             # .item() allows for numpy.int to be cast in Python 3
-            tauhatdec = np.array(fmap(lambda x: x+Decimal(rollval.item()),
-                                      tauhatdec))
+            tauhatdec = np.array([t + Decimal(rollval.item()) for t in tauhatdec])
 
             #print tauhatdec
             tauhat = tauhatdec * Decimal(dt)/Decimal(86400) #day units
@@ -1704,7 +1700,7 @@ class Archive(object):
                 P0 = self.polyco.calculatePeriod()
                 return P0
             return
-        
+
             #print P0,self.params.getPeriod()
             if np.abs(P0) < 1e-5: #Problem with large DT POLYCO values?
                 return self.params.getPeriod()
@@ -1809,13 +1805,13 @@ class Archive(object):
     def getCenterFrequency(self, weighted=False):
         """Returns the center frequency"""
         if weighted:
-            return np.sum(self.freq*self.weights)/np.sum(self.weights)
+            return np.nansum(self.freq*self.weights)/np.nansum(self.weights)
         if "HISTORY" in self.keys:
             return self.history.getLatest('CTR_FREQ')
         else:
             return self.header['OBSFREQ'] #perhaps do an unweighted version from DAT_FREQ?
     getCenterFreq = getCenterFrequency
-        
+
     def getFrequencyUnit(self):
         for key in self.subintheader.keys():
             if "TTYPE" in key:
@@ -1830,8 +1826,8 @@ class Archive(object):
         return None
 
     def getDataUnit(self):
-        """ Returns the `intensity' unit of the data 
-        Should also try to look at the 
+        """ Returns the `intensity' unit of the data
+        Should also try to look at the
         """
         for key in self.subintheader.keys():
             if "TTYPE" in key:
@@ -1844,7 +1840,7 @@ class Archive(object):
     getIntensityUnit = getDataUnit
     getFluxDensityUnit = getDataUnit
     getFluxUnit = getDataUnit
-        
+
     def getTelescope(self):
         """Returns the telescope name"""
         return self.header['TELESCOP']
