@@ -13,25 +13,27 @@ import scipy.stats as stats
 import scipy.signal as signal
 import pypulse.utils as u
 
-get_toa = u.get_toa3 #try this one
+#get_toa = u.get_toa3 #try this one
+get_toa = u.get_toa
 
 #ACF=lambda p: np.correlate(p, p, "full") #no longer used
 
-'''
-data : data array
-mpw : Main pulse window
-ipw : Interpulse window
-opw : off pulse window
-prepare : Process data according to some metrics (deprecated)
-align : Roll pulse by this amount
-period : Set pulse period
-windowsize: Override mpw, ipw, opw, define an offpulse window about the minimum of this size
-'''
+
 
 
 class SinglePulse(object):
     def __init__(self, data, mpw=None, ipw=None, opw=None, prepare=False,
-                 align=None, period=None, windowsize=None):
+                 align=None, period=None, windowsize=None, istemplate=False):
+        '''
+        data : data array
+        mpw : Main pulse window
+        ipw : Interpulse window
+        opw : off pulse window
+        prepare : Process data according to some metrics (deprecated)
+        align : Roll pulse by this amount
+        period : Set pulse period
+        windowsize: Override mpw, ipw, opw, define an offpulse window about the minimum of this size
+        '''
         if len(np.shape(data)) != 1:
             raise IndexError("SinglePulse received incorrect data shape")
 
@@ -49,7 +51,9 @@ class SinglePulse(object):
         self.bins = np.arange(self.nbins)
         self.phases = np.arange(self.nbins, dtype=np.float)/self.nbins
 
-        if windowsize is not None:
+        if windowsize is not None or (mpw is None and opw is None and windowsize is None):
+            # if either windowsize is set or nothing is set,
+            # automatically calculate
             self.calcOffpulseWindow(windowsize)
 
         elif opw is None:
@@ -85,6 +89,13 @@ class SinglePulse(object):
         if np.all(self.data == self.data[0]) or np.all(np.isnan(self.data)):
             self.null = True
 
+        if istemplate:
+            # precalculate Fourier Transform
+            self.calcFT()
+        else:
+            self.yfft = None
+            self.fs = None
+
     def interpulse_align(self):
         """
         Align the pulse such that the main pulse is at phase=0.25
@@ -93,6 +104,10 @@ class SinglePulse(object):
         self.data = np.roll(u.center_max(self.data), -len(self.data)//4)
         return self
 
+
+    def __len__(self):
+        return self.nbins
+
     def center_align(self):
         """
         Align the pulse such that the peak is in the center
@@ -133,12 +148,12 @@ class SinglePulse(object):
             factor = self.getPeriod()/self.getNbins()
         return factor*dbin
 
-    def getWeff(self, fourier=False, sumonly=False, timeunits=True):
+    def getWeff(self, fourier=False, sumonly=False):
         """
         Calculate the effective width of the pulse
         """
-        if not timeunits or self.getPeriod() is None:
-            return None
+        if self.getPeriod() is None:
+            raise ValueError("Cannot calculate Weff; period is not set")
         P = self.getPeriod()
         N = self.getNbins()
         U = u.normalize(self.data, simple=True) #remove baseline?
@@ -179,18 +194,27 @@ class SinglePulse(object):
     def calcOffpulseWindow(self, windowsize=None):
         # Find minimum in the area
         if windowsize is None:
-            windowsize = self.nbins/8
+            windowsize = self.nbins//8
 
         integral = np.zeros_like(self.data)
         for i in self.bins:
             win = np.arange(i-windowsize//2, i+windowsize//2) % self.nbins
             integral[i] = np.trapz(self.data[win])
         minind = np.argmin(integral)
-        self.opw = np.arange(minind-windowsize//2, minind+windowsize//2+1)
+        self.opw = np.arange(minind-windowsize//2, minind+windowsize//2)
         self.opw = self.opw % self.nbins
         self.mpw = self.bins[np.logical_not(np.in1d(self.bins, self.opw))]
         return self.opw
 
+    def calcFT(self):
+        """ 
+        Calculate the Fourier transform and frequencies 
+        Useful for pre-calculating a template FT
+        """
+        self.yfft = np.fft.rfft(self.data)
+        self.fs = np.fft.rfftfreq(self.nbins)
+        return self.fs, self.yfft
+
     ### Get each of the pulse components, if they exist
     def getMainpulse(self):
         if self.mpw is None:
@@ -260,17 +284,18 @@ class SinglePulse(object):
         return u.zct(self.getOffpulse(), full=True, meansub=True)
 
     # todo: include a goodness-of-fit flag (gof) as a measure of the residuals.
-    def fitPulse(self, template, fixedphase=False, rms_baseline=None):
+    def fitPulse(self, template, fixedphase=False, rms_baseline=None, remove_baseline=True):
         """
         Returns tauccf, tauhat, bhat, sigma_Tau, sigma_b, snr, rho
-        """
-        if isinstance(template, SinglePulse):
-            template = template.data
+        """                
         if self.null or len(template) != self.getNbins():
             return None
-        if rms_baseline is None:
+        #if rms_baseline is None:
+        if remove_baseline:
             self.remove_baseline()
-        if fixedphase: #just return S/N
+        if fixedphase: #just return S/N, this should be deprecated
+            if isinstance(template, SinglePulse):
+                template = template.data
             p0 = [np.max(self.data)]
             #conversion to np.float64 fixes bug with Jacobian inversion
             p1, cov, infodict, mesg, ier = \
@@ -308,7 +333,7 @@ class SinglePulse(object):
             return self
         return x
 
-    def spline_smoothing(self, sigma=None, lam=None, **kwargs):
+    def spline_smoothing(self, sigma=None, lam=None, save=False, **kwargs):
         """ Cubic Spline Interplation
         sigma: phase bin error bars
         lam: (0, 1], 1 = maximize fitting through control points (knots),
@@ -484,12 +509,16 @@ class SinglePulse(object):
         ytemp = np.roll(ytemp, -shift)
         ytemp /= np.max(ytemp)
 
+        if save:
+            self.data = ytemp
         return ytemp
     splineSmoothing = spline_smoothing
 
+
     def component_fitting(self, mode='gaussian', nmax=10, full=False,
                           minamp=None, alpha=0.05, allownegative=False,
-                          verbose=False, save=False):
+                          verbose=False, plot=False, save=False,
+                          filename=None):
         '''
         Fitting to phases is much more numerically stable for von mises function
         '''
@@ -507,7 +536,20 @@ class SinglePulse(object):
         residsA = self.data - fitfunc(pfit, self.phases)
         RSS_funcA = np.sum(residsA**2)
 
-        def doreturn(save=False):
+        def doreturn(plot=False, filename=None, save=False):
+            if plot:
+                componentfunc = eval("u.%s"%mode)
+                plt.plot(self.phases, self.data)
+                for i in range(0, len(pfit), 3):
+                    plt.plot(self.phases, componentfunc(self.phases, pfit[i], pfit[i+1], pfit[i+2]), '--', color='0.50', alpha=0.5)
+                plt.plot(self.phases, fitfunc(pfit, self.phases), 'k')
+                plt.xlabel('Phase')
+                plt.ylabel('Intensity')
+                plt.tight_layout()
+                if filename is not None:
+                    plt.savefig(filename)
+                plt.show()
+                
             if save:
                 self.data = fitfunc(pfit, self.phases)
             if full:
@@ -523,7 +565,7 @@ class SinglePulse(object):
             RSS_funcB = np.sum(residsB**2)
 
             if minamp is not None and np.all(residsB < MAX):
-                return doreturn(save=save)
+                return doreturn(plot=plot, filename=filename, save=save)
 
             # F-test
             F = ((RSS_funcA-RSS_funcB)/(nparamB-nparamA))/(RSS_funcB/(N-nparamB-1))
@@ -532,11 +574,11 @@ class SinglePulse(object):
             #print F, RSS_funcA, RSS_funcB, nparamA, nparamB, p_value
             if p_value > alpha: # if p_value < alpha,  then B is significant, so keep going
             # Replace old values
-                return doreturn(save=save)
+                return doreturn(plot=plot, filename=filename, save=save)
             nparamA = nparamB
             residsA = residsB
             RSS_funcA = RSS_funcB
-        return doreturn(save=save)
+        return doreturn(plot=plot, filename=filename, save=save)
 
         '''
         n = 1
@@ -572,11 +614,14 @@ class SinglePulse(object):
         return self.component_fitting(mode='vonmises', **kwargs)
     vonMises_smoothing = vonmises_smoothing
 
-    def savgol_smoothing(self, window_length=11, polyorder=5):
+    def savgol_smoothing(self, window_length=11, polyorder=5, save=False):
         """
         Savitzky-Golay filter smoothing
         """
-        return signal.savgol_filter(self.data, window_length, polyorder)
+        retval = signal.savgol_filter(self.data, window_length, polyorder)
+        if save:
+            self.data = retval
+        return retval
 
     def smooth(self, mode='vonmises', sigma=None, lam=None,
                window_length=11, polyorder=5, **kwargs):
@@ -683,6 +728,24 @@ class SinglePulse(object):
             plt.savefig("cleanimages/%s_clean.png"%name)
             plt.close()
 
+
+    def addNoise(self, sn, save=False):
+        """ 
+        Add noise to the pulse such that it will roughly 
+        have an S/N given by sn 
+        """
+        
+        minimum = np.mean(self.getOffpulse())
+        maximum = np.max(self.data)
+        amplitude = maximum - minimum
+        sigma = amplitude/sn
+        
+        retval = self.data + np.random.normal(0, sigma, self.nbins)
+        if save:
+            self.data = retval
+        return retval
+        
+
     def getPeriod(self):
         """ Return the pulse period """
         return self.period
@@ -696,6 +759,9 @@ class SinglePulse(object):
         """ Return the time unit of one phase bin """
         if self.getPeriod() is not None:
             return self.getPeriod()/self.getNbins()
+        else:
+            raise ValueError("Cannot get bin time; period is not set")
+
 
     def plot(self, show=True):
         """
